@@ -10,12 +10,14 @@ from functools import reduce
 import matplotlib.pyplot as plt
 from matplotlib.patches import Wedge
 from matplotlib.collections import PatchCollection
+import matplotlib.colors as colors
 import matplotlib.transforms as transforms
 import numpy as np
 import pandas as pd
 import time
 from copy import deepcopy
 import os
+from scipy.ndimage import gaussian_filter
 
 import matplotlib as mpl
 mpl.rcParams.update(mpl.rcParamsDefault)
@@ -574,7 +576,7 @@ class Band:
         return kdist
 
 
-    def _get_kticks(self, ax):
+    def _get_kticks(self, ax, wave_vectors, vlinecolor):
         """
         This function extracts the kpoint labels and index locations for a regular
         band structure calculation (non unfolded).
@@ -598,15 +600,15 @@ class Band:
 
         kpts_labels = kpts_labels[index]
         kpoints_index = list(kpoints_index[index])
-        kpoints_index = ax.lines[0].get_xdata()[kpoints_index]
+        #  kpoints_index = ax.lines[0].get_xdata()[kpoints_index]
 
         for k in kpoints_index:
-            ax.axvline(x=k, color='black', alpha=0.7, linewidth=0.5)
+            ax.axvline(x=wave_vectors[k], color=vlinecolor, alpha=0.7, linewidth=0.5)
         
-        ax.set_xticks(kpoints_index)
+        ax.set_xticks([wave_vectors[k] for k in kpoints_index])
         ax.set_xticklabels(kpts_labels)
 
-    def _get_kticks_hse(self, ax, kpath):
+    def _get_kticks_hse(self, ax, kpath, vlinecolor):
         structure = self.poscar.structure
         kpath_obj = HighSymmKpath(structure)
         kpath_labels = np.array(list(kpath_obj._kpath['kpoints'].keys()))
@@ -628,19 +630,22 @@ class Band:
         kpath = [f'${k}$' if k != 'G' else '$\\Gamma$' for k in kpath]
 
         for k in kpoints_index:
-            ax.axvline(x=k, color='black', alpha=0.7, linewidth=0.5)
+            ax.axvline(x=k, color=vlinecolor, alpha=0.7, linewidth=0.5)
 
         plt.xticks(kpoints_index, kpath)
 
-    def _get_kticks_unfold(self, ax, wave_vectors):
-        kpath = [
-            f'${k}$' if k != 'G' else '$\\Gamma$' for k in self.kpath.upper().strip()
-        ]
+    def _get_kticks_unfold(self, ax, wave_vectors, vlinecolor):
+        if type(self.kpath) == str:
+            kpath = [
+                f'${k}$' if k != 'G' else '$\\Gamma$' for k in self.kpath.upper().strip()
+            ]
+        elif type(self.kpath) == list:
+            kpath = self.kpath
 
         kpoints_index = [0] + [(self.n * i) for i in range(1, len(self.kpath))]
 
         for k in kpoints_index:
-            ax.axvline(x=wave_vectors[k], color='black', alpha=0.7, linewidth=0.5)
+            ax.axvline(x=wave_vectors[k], color=vlinecolor, alpha=0.7, linewidth=0.5)
 
         ax.set_xticks(wave_vectors[kpoints_index])
         ax.set_xticklabels(kpath)
@@ -648,7 +653,7 @@ class Band:
 
     def _filter_bands(self, erange):
         eigenvalues = self.eigenvalues
-        where = (eigenvalues >= np.min(erange)) & (eigenvalues <= np.max(erange))
+        where = (eigenvalues >= np.min(erange) - 1) & (eigenvalues <= np.max(erange) + 1)
         is_true = np.sum(np.isin(where, True), axis=1)
         bands_in_plot = is_true > 0
 
@@ -693,7 +698,70 @@ class Band:
             handletextpad=0.1,
         )
 
-    def plot_plain(self, ax, color='black', erange=[-6,6], linewidth=1.25, scale_factor=20, linestyle='-'):
+    def _heatmap(
+        self,
+        ax,
+        wave_vectors,
+        eigenvalues,
+        weights,
+        sigma,
+        cmap,
+        bins,
+        projection=None,
+        powernorm=True,
+        gamma=0.5
+    ):
+        eigenvalues_ravel = np.ravel(eigenvalues)
+        wave_vectors_tile = np.tile(wave_vectors, eigenvalues.shape[0])
+
+        if projection is not None:
+            if len(np.squeeze(projection).shape) == 2:
+                weights *= np.squeeze(projection)
+            else:
+                weights *= np.sum(np.squeeze(projection), axis=2)
+
+        weights_ravel = np.ravel(weights)
+
+        data = np.histogram2d(
+            wave_vectors_tile,
+            eigenvalues_ravel,
+            bins=bins,
+            weights=weights_ravel,
+        )[0]
+
+        data = gaussian_filter(data, sigma=sigma)
+        if powernorm:
+            norm = colors.PowerNorm(gamma=gamma, vmin=np.min(data), vmax=np.max(data))
+        else:
+            norm = colors.Normalize(vmin=np.min(data), vmax=np.max(data))
+
+        ax.pcolormesh(
+            np.linspace(np.min(wave_vectors), np.max(wave_vectors), bins),
+            np.linspace(np.min(eigenvalues), np.max(eigenvalues), bins),
+            data.T,
+            shading='gouraud',
+            cmap=cmap,
+            norm=norm,
+        )
+
+
+    def plot_plain(
+        self,
+        ax,
+        color='black',
+        erange=[-6,6],
+        linewidth=1.25,
+        scale_factor=20,
+        linestyle='-',
+        heatmap=False,
+        bins=400,
+        sigma=3,
+        cmap='hot',
+        vlinecolor='black',
+        powernorm=False,
+        gamma=0.5,
+        projection=None,
+    ):
         """
         This function plots a plain band structure.
 
@@ -703,12 +771,9 @@ class Band:
             linewidth (float): Line width of the band structure lines
             linestyle (str): Line style of the bands
         """
-
         bands_in_plot = self._filter_bands(erange=erange)
         eigenvalues = self.eigenvalues[bands_in_plot]
         wave_vectors = self._get_k_distance()
-        #  if self.unfold:
-            #  wave_vectors = (wave_vectors / np.max(wave_vectors)) * 5
         eigenvalues_ravel = np.ravel(np.c_[eigenvalues, np.empty(eigenvalues.shape[0]) * np.nan])
         wave_vectors_tile = np.tile(np.append(wave_vectors, np.nan), eigenvalues.shape[0])
 
@@ -716,14 +781,29 @@ class Band:
             spectral_weights = self.spectral_weights[bands_in_plot]
             spectral_weights = spectral_weights / np.max(spectral_weights)
             spectral_weights_ravel = np.ravel(np.c_[spectral_weights, np.empty(spectral_weights.shape[0]) * np.nan])
-            ax.scatter(
-                wave_vectors_tile,
-                eigenvalues_ravel,
-                c=color,
-                ec=None,
-                s=scale_factor * spectral_weights_ravel,
-                zorder=0,
-            )
+
+            if heatmap:
+                self._heatmap(
+                    ax=ax,
+                    wave_vectors=wave_vectors,
+                    eigenvalues=eigenvalues,
+                    weights=spectral_weights,
+                    sigma=sigma,
+                    cmap=cmap,
+                    bins=bins,
+                    projection=projection,
+                    powernorm=powernorm,
+                    gamma=gamma,
+                )
+            else:
+                ax.scatter(
+                    wave_vectors_tile,
+                    eigenvalues_ravel,
+                    c=color,
+                    ec=None,
+                    s=scale_factor * spectral_weights_ravel,
+                    zorder=0,
+                )
         else:
             ax.plot(
                 wave_vectors_tile,
@@ -735,16 +815,33 @@ class Band:
             )
 
         if self.hse:
-            self._get_kticks_hse(ax=ax, kpath=self.kpath)
+            self._get_kticks_hse(ax=ax, kpath=self.kpath, vlinecolor=vlinecolor)
         elif self.unfold:
-            self._get_kticks_unfold(ax=ax, wave_vectors=wave_vectors)
+            self._get_kticks_unfold(ax=ax, wave_vectors=wave_vectors, vlinecolor=vlinecolor)
         else:
-            self._get_kticks(ax=ax)
+            self._get_kticks(ax=ax, wave_vectors=wave_vectors, vlinecolor=vlinecolor)
 
         ax.set_xlim(0, np.max(wave_vectors))
 
 
-    def _plot_projected_general(self, ax, projected_data, colors, scale_factor=5, erange=[-6,6], display_order=None, linewidth=0.75, band_color='black'):
+    def _plot_projected_general(
+        self,
+        ax,
+        projected_data,
+        colors,
+        scale_factor=5,
+        erange=[-6,6],
+        display_order=None,
+        linewidth=0.75,
+        band_color='black',
+        heatmap=False,
+        bins=400,
+        sigma=3,
+        cmap='hot',
+        vlinecolor='black',
+        powernorm=False,
+        gamma=0.5,
+    ):
         """
         This is a general method for plotting projected data
 
@@ -762,61 +859,96 @@ class Band:
             band_color = [(0.9,0.9,0.9)]
             scale_factor = scale_factor * 4
 
-        self.plot_plain(ax=ax, linewidth=linewidth, color=band_color, erange=erange)
-
         bands_in_plot = self._filter_bands(erange=erange)
         projected_data = projected_data[bands_in_plot]
         projected_data = projected_data / np.max(projected_data)
         wave_vectors = self._get_k_distance()
         eigenvalues = self.eigenvalues[bands_in_plot]
 
-        if self.unfold:
-            spectral_weights = self.spectral_weights[bands_in_plot]
-            spectral_weights = spectral_weights / np.max(spectral_weights)
-            K_indices = np.array(self.K_indices[0], dtype=int)
-            projected_data = projected_data[:, K_indices, :]
-            spectral_weights_ravel = np.repeat(np.ravel(spectral_weights), projected_data.shape[-1])
+        if heatmap:
+            if self.unfold:
+                K_indices = np.array(self.K_indices[0], dtype=int)
+                projected_data = projected_data[:, K_indices, :]
 
-        projected_data_ravel = np.ravel(projected_data)
-        wave_vectors_tile = np.tile(
-            np.repeat(wave_vectors, projected_data.shape[-1]), projected_data.shape[0]
+        self.plot_plain(
+            ax=ax,
+            linewidth=linewidth,
+            color=band_color,
+            erange=erange,
+            heatmap=heatmap,
+            sigma=sigma,
+            cmap=cmap,
+            bins=bins,
+            vlinecolor=vlinecolor,
+            projection=projected_data,
         )
-        eigenvalues_tile = np.repeat(np.ravel(eigenvalues), projected_data.shape[-1])
-        colors_tile = np.tile(colors, np.prod(projected_data.shape[:-1]))
 
-        if display_order is None:
-            pass
-        else:
-            sort_index = np.argsort(projected_data_ravel)
+        if not heatmap:
+            if self.unfold:
+                spectral_weights = self.spectral_weights[bands_in_plot]
+                spectral_weights = spectral_weights / np.max(spectral_weights)
+                K_indices = np.array(self.K_indices[0], dtype=int)
+                projected_data = projected_data[:, K_indices, :]
+                spectral_weights_ravel = np.repeat(np.ravel(spectral_weights), projected_data.shape[-1])
 
-            if display_order == 'all':
-                sort_index = sort_index[::-1]
+            projected_data_ravel = np.ravel(projected_data)
+            wave_vectors_tile = np.tile(
+                np.repeat(wave_vectors, projected_data.shape[-1]), projected_data.shape[0]
+            )
+            eigenvalues_tile = np.repeat(np.ravel(eigenvalues), projected_data.shape[-1])
+            colors_tile = np.tile(colors, np.prod(projected_data.shape[:-1]))
 
-            wave_vectors_tile = wave_vectors_tile[sort_index]
-            eigenvalues_tile = eigenvalues_tile[sort_index]
-            colors_tile = colors_tile[sort_index]
-            projected_data_ravel = projected_data_ravel[sort_index]
+            if display_order is None:
+                pass
+            else:
+                sort_index = np.argsort(projected_data_ravel)
+
+                if display_order == 'all':
+                    sort_index = sort_index[::-1]
+
+                wave_vectors_tile = wave_vectors_tile[sort_index]
+                eigenvalues_tile = eigenvalues_tile[sort_index]
+                colors_tile = colors_tile[sort_index]
+                projected_data_ravel = projected_data_ravel[sort_index]
+
+                if self.unfold:
+                    spectral_weights_ravel = spectral_weights_ravel[sort_index]
 
             if self.unfold:
-                spectral_weights_ravel = spectral_weights_ravel[sort_index]
+                s = scale_factor * projected_data_ravel * spectral_weights_ravel
+                ec = None
+            else:
+                s = scale_factor * projected_data_ravel
+                ec = colors_tile
 
-        if self.unfold:
-            s = scale_factor * projected_data_ravel * spectral_weights_ravel
-            ec = None
-        else:
-            s = scale_factor * projected_data_ravel
-            ec = colors_tile
-
-        ax.scatter(
-            wave_vectors_tile,
-            eigenvalues_tile,
-            c=colors_tile,
-            ec=ec,
-            s=s,
-            zorder=100,
-        )
+            ax.scatter(
+                wave_vectors_tile,
+                eigenvalues_tile,
+                c=colors_tile,
+                ec=ec,
+                s=s,
+                zorder=100,
+            )
         
-    def plot_orbitals(self, ax, orbitals, scale_factor=5, erange=[-6,6], display_order=None, color_list=None, legend=True, linewidth=0.75, band_color='black'):
+    def plot_orbitals(
+        self,
+        ax,
+        orbitals,
+        scale_factor=5,
+        erange=[-6,6],
+        display_order=None,
+        color_list=None,
+        legend=True,
+        linewidth=0.75,
+        band_color='black',
+        heatmap=False,
+        bins=400,
+        sigma=3,
+        cmap='hot',
+        vlinecolor='black',
+        powernorm=False,
+        gamma=0.5,
+    ):
         """
         This function plots the projected band structure of given orbitals summed across all atoms on a given axis.
 
@@ -866,14 +998,37 @@ class Band:
             erange=erange,
             display_order=display_order,
             linewidth=linewidth,
-            band_color=band_color
+            band_color=band_color,
+            heatmap=heatmap,
+            bins=bins,
+            sigma=sigma,
+            cmap=cmap,
+            vlinecolor=vlinecolor,
         )
 
         if legend:
             self._add_legend(ax, names=[self.orbital_labels[i] for i in orbitals], colors=colors)
 
 
-    def plot_spd(self, ax, scale_factor=5, orbitals='spd', erange=[-6,6], display_order=None, color_dict=None, legend=True, linewidth=0.75, band_color='black'):
+    def plot_spd(
+        self,
+        ax,
+        scale_factor=5,
+        orbitals='spd',
+        erange=[-6,6],
+        display_order=None,
+        color_dict=None,
+        legend=True,
+        linewidth=0.75,
+        band_color='black',
+        heatmap=False,
+        bins=400,
+        sigma=3,
+        cmap='hot',
+        vlinecolor='black',
+        powernorm=False,
+        gamma=0.5,
+    ):
         """
         This function plots the s, p, d projected band structure onto a given axis
 
@@ -914,7 +1069,12 @@ class Band:
             erange=erange,
             display_order=display_order,
             linewidth=linewidth,
-            band_color=band_color
+            band_color=band_color,
+            heatmap=heatmap,
+            bins=bins,
+            sigma=sigma,
+            cmap=cmap,
+            vlinecolor=vlinecolor,
         )
 
         if legend:
@@ -922,7 +1082,25 @@ class Band:
 
 
 
-    def plot_atoms(self, ax, atoms, scale_factor=5, erange=[-6,6], display_order=None, color_list=None, legend=True, linewidth=0.75, band_color='black'):
+    def plot_atoms(
+        self,
+        ax,
+        atoms,
+        scale_factor=5,
+        erange=[-6,6],
+        display_order=None,
+        color_list=None,
+        legend=True,
+        linewidth=0.75,
+        band_color='black',
+        heatmap=False,
+        bins=400,
+        sigma=3,
+        cmap='hot',
+        vlinecolor='black',
+        powernorm=False,
+        gamma=0.5,
+    ):
         """
         This function plots the projected band structure of given atoms summed across all orbitals on a given axis.
 
@@ -951,14 +1129,37 @@ class Band:
             erange=erange,
             display_order=display_order,
             linewidth=linewidth,
-            band_color=band_color
+            band_color=band_color,
+            heatmap=heatmap,
+            bins=bins,
+            sigma=sigma,
+            cmap=cmap,
+            vlinecolor=vlinecolor,
         )
 
         if legend:
             self._add_legend(ax, names=atoms, colors=colors)
 
 
-    def plot_atom_orbitals(self, ax, atom_orbital_dict, scale_factor=5, erange=[-6,6], display_order=None, color_list=None, legend=True, linewidth=0.75, band_color='black'):
+    def plot_atom_orbitals(
+        self,
+        ax,
+        atom_orbital_dict,
+        scale_factor=5,
+        erange=[-6,6],
+        display_order=None,
+        color_list=None,
+        legend=True,
+        linewidth=0.75,
+        band_color='black',
+        heatmap=False,
+        bins=400,
+        sigma=3,
+        cmap='hot',
+        vlinecolor='black',
+        powernorm=False,
+        gamma=0.5,
+    ):
         """
         This function plots the projected band structure of individual orbitals on a given axis.
 
@@ -1004,7 +1205,12 @@ class Band:
             erange=erange,
             display_order=display_order,
             linewidth=linewidth,
-            band_color=band_color
+            band_color=band_color,
+            heatmap=heatmap,
+            bins=bins,
+            sigma=sigma,
+            cmap=cmap,
+            vlinecolor=vlinecolor,
         )
 
         if legend:
@@ -1014,7 +1220,25 @@ class Band:
                 colors=colors
             )
 
-    def plot_atom_spd(self, ax, atom_spd_dict, scale_factor=5, erange=[-6,6], display_order=None, color_list=None, legend=True, linewidth=0.75, band_color='black'):
+    def plot_atom_spd(
+        self,
+        ax,
+        atom_spd_dict,
+        scale_factor=5,
+        erange=[-6,6],
+        display_order=None,
+        color_list=None,
+        legend=True,
+        linewidth=0.75,
+        band_color='black',
+        heatmap=False,
+        bins=400,
+        sigma=3,
+        cmap='hot',
+        vlinecolor='black',
+        powernorm=False,
+        gamma=0.5,
+    ):
         """
         This function plots the projected band structure on the s, p, and d orbitals for each specified atom in the calculated structure.
 
@@ -1061,7 +1285,12 @@ class Band:
             erange=erange,
             display_order=display_order,
             linewidth=linewidth,
-            band_color=band_color
+            band_color=band_color,
+            heatmap=heatmap,
+            bins=bins,
+            sigma=sigma,
+            cmap=cmap,
+            vlinecolor=vlinecolor,
         )
 
         if legend:
@@ -1073,7 +1302,25 @@ class Band:
 
 
 
-    def plot_elements(self, ax, elements, scale_factor=5, erange=[-6,6], display_order=None, color_list=None, legend=True, linewidth=0.75, band_color='black'):
+    def plot_elements(
+        self,
+        ax,
+        elements,
+        scale_factor=5,
+        erange=[-6,6],
+        display_order=None,
+        color_list=None,
+        legend=True,
+        linewidth=0.75,
+        band_color='black',
+        heatmap=False,
+        bins=400,
+        sigma=3,
+        cmap='hot',
+        vlinecolor='black',
+        powernorm=False,
+        gamma=0.5,
+    ):
         """
         This function plots the projected band structure on specified elements in the calculated structure
 
@@ -1102,14 +1349,37 @@ class Band:
             erange=erange,
             display_order=display_order,
             linewidth=linewidth,
-            band_color=band_color
+            band_color=band_color,
+            heatmap=heatmap,
+            bins=bins,
+            sigma=sigma,
+            cmap=cmap,
+            vlinecolor=vlinecolor,
         )
 
         if legend:
             self._add_legend(ax, names=elements, colors=colors)
 
 
-    def plot_element_orbitals(self, ax, element_orbital_dict, scale_factor=5, erange=[-6,6], display_order=None, color_list=None, legend=True, linewidth=0.75, band_color='black'):
+    def plot_element_orbitals(
+        self,
+        ax,
+        element_orbital_dict,
+        scale_factor=5,
+        erange=[-6,6],
+        display_order=None,
+        color_list=None,
+        legend=True,
+        linewidth=0.75,
+        band_color='black',
+        heatmap=False,
+        bins=400,
+        sigma=3,
+        cmap='hot',
+        vlinecolor='black',
+        powernorm=False,
+        gamma=0.5,
+    ):
         """
         this function plots the projected band structure on chosen orbitals for each specified element in the calculated structure.
 
@@ -1151,7 +1421,12 @@ class Band:
             erange=erange,
             display_order=display_order,
             linewidth=linewidth,
-            band_color=band_color
+            band_color=band_color,
+            heatmap=heatmap,
+            bins=bins,
+            sigma=sigma,
+            cmap=cmap,
+            vlinecolor=vlinecolor,
         )
 
         if legend:
@@ -1161,7 +1436,25 @@ class Band:
                 colors=colors
             )
 
-    def plot_element_spd(self, ax, element_spd_dict, scale_factor=5, erange=[-6,6], display_order=None, color_list=None, legend=True, linewidth=0.75, band_color='black'):
+    def plot_element_spd(
+        self,
+        ax,
+        element_spd_dict,
+        scale_factor=5,
+        erange=[-6,6],
+        display_order=None,
+        color_list=None,
+        legend=True,
+        linewidth=0.75,
+        band_color='black',
+        heatmap=False,
+        bins=400,
+        sigma=3,
+        cmap='hot',
+        vlinecolor='black',
+        powernorm=False,
+        gamma=0.5,
+    ):
         """
         This function plots the projected band structure on the s, p, and d orbitals for each specified element in the calculated structure.
 
@@ -1210,7 +1503,12 @@ class Band:
             erange=erange,
             display_order=display_order,
             linewidth=linewidth,
-            band_color=band_color
+            band_color=band_color,
+            heatmap=heatmap,
+            bins=bins,
+            sigma=sigma,
+            cmap=cmap,
+            vlinecolor=vlinecolor,
         )
 
         if legend:
@@ -1222,19 +1520,31 @@ class Band:
 
 
 if __name__ == "__main__":
-    M = [[-1,1,0],[-1,-1,1],[0,0,1]]
+    M = [
+        [0,1,-1],
+        [1,-1,0],
+        [-14,-14,-14]
+    ]
+
     high_symm_points = [
-        [0.5, 0.5, 0],
+        [2/3, 1/3, 1/3],
         [0.0, 0.0, 0],
-        [0.5, 0.5, 0]
-    ] 
+        [2/3, 1/3, 1/3],
+    ]
+
+    #  high_symm_points = [
+        #  [0.1, 0.1, 0],
+        #  [0.0, 0.0, 0],
+        #  [0.1, 0.1, 0],
+    #  ]
+
     band = Band(
-        folder="../../vaspvis_data/band-unfold",
-        projected=True,
+        folder="../../vaspvis_data/bandMGM",
+        projected=False,
         unfold=True,
-        kpath='XGX',
-        high_symm_points=high_symm_points, 
-        n=30,
+        kpath='AGA',
+        high_symm_points=high_symm_points,
+        n=40,
         M=M,
     )
     fig, ax = plt.subplots(figsize=(3,4), dpi=300)
@@ -1242,12 +1552,17 @@ if __name__ == "__main__":
     #  band.plot_plain(ax=ax, color=[(0.9,0.9,0.9)])
     #  band.plot_spd(ax=ax, orbitals='sd', display_order='all', scale_factor=35, erange=[-5,0])
     #  band.plot_orbitals(ax=ax, scale_factor=35, orbitals=range(8), display_order=None)
-    band.plot_atoms( 
+    band.plot_plain( 
         ax=ax,
-        atoms=[0,1],
-        display_order='dominant',
         scale_factor=20,
-        erange=[-5,0],
+        erange=[-4,0.5],
+        heatmap=True,
+        cmap='hot',
+        bins=800,
+        sigma=3,
+        powernorm=False,
+        gamma=0.5,
+        vlinecolor='white'
     )
     #  ax.set_aspect(3, adjustable='datalim')
     end = time.time()
@@ -1255,9 +1570,9 @@ if __name__ == "__main__":
     ax.set_ylabel('$E - E_{F}$ $(eV)$', fontsize=6)
     ax.tick_params(labelsize=6, length=2.5)
     ax.tick_params(axis='x', length=0)
-    ax.set_ylim(-5,0)
+    ax.set_ylim(-4, 0.5)
     plt.tight_layout(pad=0.2)
-    plt.savefig('unfold_spd_dominant.png')
+    plt.savefig('heatmap_powernorm.png')
         
         
 
